/*
 * Copyright (c) eHealth
 */
package be.ehealth.businessconnector.test.dmg.mappers;

import be.ehealth.businessconnector.dmg.mappers.CommonInputMapper;
import org.junit.Test;
import org.mapstruct.factory.Mappers;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import be.ehealth.business.mycarenetdomaincommons.domain.CareProvider;
import be.ehealth.business.mycarenetdomaincommons.domain.CommonInput;
import be.ehealth.business.mycarenetdomaincommons.domain.McnPackageInfo;
import be.ehealth.business.mycarenetdomaincommons.domain.Nihii;
import be.ehealth.business.mycarenetdomaincommons.domain.Origin;
import be.ehealth.technicalconnector.utils.ConnectorIOUtils;
import be.ehealth.technicalconnector.utils.MarshallerHelper;
import be.fgov.ehealth.globalmedicalfile.core.v1.CommonInputType;
import be.fgov.ehealth.technicalconnector.tests.utils.XmlAsserter;


/**
 * Test for {@link CommonInputMapper}
 * 
 * @author EHP
 * 
 */
public class CommonInputMapperTest {

    private static final Logger LOG = LoggerFactory.getLogger(CommonInputMapperTest.class);

    @Test
    public void testCommonInputMapping() throws Exception {
        MarshallerHelper<CommonInputType, CommonInputType> helper = new MarshallerHelper<CommonInputType, CommonInputType>(CommonInputType.class, CommonInputType.class);
        CommonInputType commonInputExample = helper.toObject(ConnectorIOUtils.getBytes(ConnectorIOUtils.getResourceAsStream("/examples/commonInput-dmg.xml")));

        Boolean istest = commonInputExample.getRequest().isIsTest();
        String ref = new String(commonInputExample.getInputReference());
        Nihii nihii = new Nihii(commonInputExample.getOrigin().getCareProvider().getNihii().getQuality(), commonInputExample.getOrigin().getCareProvider().getNihii().getValue().getValue());
        CareProvider careProvider = new CareProvider(nihii);
        Origin origin = new Origin(new McnPackageInfo(commonInputExample.getOrigin().getPackage().getLicense().getUsername(), commonInputExample.getOrigin().getPackage().getLicense().getPassword(), commonInputExample.getOrigin().getPackage().getName().getValue()), careProvider);
        origin.setSiteId(commonInputExample.getOrigin().getSiteID().getValue());
        CommonInput input = new CommonInput(istest, origin, ref);
        CommonInputType output = Mappers.getMapper(CommonInputMapper.class).map(input);

        LOG.debug(helper.toString(output));
        XmlAsserter.assertSimilar(helper.toString(commonInputExample), helper.toString(output));
    }

}
